@@ -7,9 +7,9 @@ use warnings;
 use Exporter qw(import);
 
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
-our $DATE = '2022-05-21'; # DATE
+our $DATE = '2022-05-22'; # DATE
 our $DIST = 'Array-Sample-WeightedRandom'; # DIST
-our $VERSION = '0.004'; # VERSION
+our $VERSION = '0.005'; # VERSION
 
 our @EXPORT_OK = qw(sample_weighted_random_with_replacement
                     sample_weighted_random_no_replacement);
@@ -33,7 +33,7 @@ sub sample_weighted_random_with_replacement {
             my $y2 = $y + $elem->[1];
             if ($x >= $y && $x < $y2) {
                 my $idx = $j;
-                push @res, $opts->{pos} ? $idx : $ary->[$idx][0];
+                push @res, $opts->{with_weight} ? $ary->[$idx] : $opts->{pos} ? $idx : $ary->[$idx][0];
                 last;
             }
             $y = $y2;
@@ -76,7 +76,7 @@ sub sample_weighted_random_with_replacement {
 
                 my $y2 = $y + $elem->[1];
                 if ($x >= $y && $x < $y2) {
-                    push @res, $opts->{pos} ? $j : $elem->[0];
+                    push @res, $opts->{with_weight} ? $elem : $opts->{pos} ? $j : $elem->[0];
                     $sum_of_weights -= $elem->[1];
                     $picked{$j}++;
                     last;
@@ -96,7 +96,7 @@ sub sample_weighted_random_with_replacement {
                 my $elem = $ary_copy[$j];
                 my $y2 = $y + $elem->[1];
                 if ($x >= $y && $x < $y2) {
-                    push @res, $opts->{pos} ? $pos[$j] : $elem->[0];
+                    push @res, $opts->{with_weight} ? $elem : $opts->{pos} ? $pos[$j] : $elem->[0];
                     $sum_of_weights -= $elem->[1];
                     splice @ary_copy, $j, 1;
                     splice @pos     , $j, 1;
@@ -130,7 +130,7 @@ Array::Sample::WeightedRandom - Sample elements randomly, with weights (with or
 
 =head1 VERSION
 
-This document describes version 0.004 of Array::Sample::WeightedRandom (from Perl distribution Array-Sample-WeightedRandom), released on 2022-05-21.
+This document describes version 0.005 of Array::Sample::WeightedRandom (from Perl distribution Array-Sample-WeightedRandom), released on 2022-05-22.
 
 =head1 SYNOPSIS
 
@@ -179,6 +179,11 @@ Default is 'copy'. Another choice is 'nocopy', which avoids creating a shallow
 slower but could save memory usage *if* your array is very very large (e.g. tens
 of millions of elements).
 
+=item * with_weight => bool
+
+If set to true, will return the original elements (values with weights) instead
+of just the values.
+
 =back
 
 The function takes an array reference (C<\@ary>) and number of samples to take
