@@ -7,9 +7,9 @@ use warnings;
 use Exporter qw(import);
 
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
-our $DATE = '2022-05-21'; # DATE
+our $DATE = '2022-05-22'; # DATE
 our $DIST = 'Array-Sample-WeightedRandom'; # DIST
-our $VERSION = '0.004'; # VERSION
+our $VERSION = '0.005'; # VERSION
 
 our @EXPORT_OK = qw(sample_weighted_random_no_replacement);
 
@@ -45,52 +45,11 @@ sub _sample_weighted_random_no_replacement {
 
         #use DD; print "D:item: "; dd $item;
         if (@res < $num_items) {
-            # we haven't collected $num_items in @res, insert item to array in a
-            # random position (adjusted by weights, so higher-weighted item will
-            # tend to be at the front)
-
-            if (!@res) {
-                #say "D:added";
-                push @res, [$item->[0], $item->[1], $i];
-            } else {
-                #say "D:inserting in random location";
-
-                # UNUSED: weighted variant. note that this variant still does
-                # not produce weighted position but biased towards the earilest
-                # elements. thus earliest elements will sstill my $x =
-                # rand($sum_of_weights_res + $item->[1]); say "D:
-                # x=rand($sum_of_weights_res + $item->[1])=$x";
-
-                #my $y = 0;
-                #my $added;
-                #for my $j (0 .. $#res) {
-                #    my $elem = $res[$j];
-                #    my $y2 = $y + $elem->[1];
-                #    #say "D:  j=$j, $x >= $y && $x < $y2 ?";
-                #    if ($x >= $y && $x < $y2) {
-                #        my $idx = $j;
-                #        #say "D:  inserted at position ".($j+1);
-                #        splice @res, $j+1, 0, [$item->[0], $item->[1], $i];
-                #        $added++;
-                #        last;
-                #    }
-                #    $y = $y2;
-                #}
-                #unless ($added) {
-                #    #say "D:  inserted at the end";
-                #    push @res, [$item->[0], $item->[1], $i];
-                #}
-
-                # USED: uniform random variant. this variant inserts randomly so
-                # does not take weight into account. weight will be taken into
-                # account during replacing
-                my $idx = rand(@res+1);
-                splice @res, $idx, 0, [$item->[0], $item->[1], $i];
-
-            }
+            # we haven't collected $num_items in @res, populate @res first.
+            #say "D:  adding";
+            push @res, [$item->[0], $item->[1], $i];
             $sum_of_weights_res     += $item->[1];
             $sum_of_inv_weights_res += 1/$item->[1];
-
         } else {
             #say "D:maybe replacing (probability: rand($sum_of_inv_weights_iterated + 1/$item->[1]) < $sum_of_inv_weights_res)";
             # we have reached $num_items in @res, probabilistically replace an
@@ -128,6 +87,9 @@ sub _sample_weighted_random_no_replacement {
     if ($opts->{shuffle}) {
         require List::Util;
         @res = List::Util::shuffle(@res);
+    } else {
+        require Array::Sample::WeightedRandom;
+        @res = Array::Sample::WeightedRandom::sample_weighted_random_no_replacement(\@res, scalar(@res), {with_weight=>1});
     }
 
     if ($opts->{pos}) {
@@ -162,7 +124,7 @@ Array::Sample::WeightedRandom::Scan - (DO NOT USE) Sample elements randomly, wit
 
 =head1 VERSION
 
-This document describes version 0.004 of Array::Sample::WeightedRandom::Scan (from Perl distribution Array-Sample-WeightedRandom), released on 2022-05-21.
+This document describes version 0.005 of Array::Sample::WeightedRandom::Scan (from Perl distribution Array-Sample-WeightedRandom), released on 2022-05-22.
 
 =head1 SYNOPSIS
 
@@ -177,8 +139,9 @@ This document describes version 0.004 of Array::Sample::WeightedRandom::Scan (fr
 
 =head1 DESCRIPTION
 
-B<DO NOT USE>. This algorithm currently produces biased results. Use
-L<Array::Sample::WeightedRandom> instead.
+B<DO NOT USE>. This algorithm currently produces biased results where the items
+at the beginning will be picked at probabiity not quite according to their
+weights. Use L<Array::Sample::WeightedRandom> instead.
 
 This module provides L</sample_weighted_random_no_replacement> which is the same
 as the one provided by L<Array::Sample::WeightedRandom> but uses the scan
